@@ -6,4 +6,5 @@ from .attr import attr
 from .list import llist
 from .tree import tree
 
-from .enums import TrainerStage
+from ..kit.beans.trainstage import TrainerStage
+from .trickitems import null, NoneItem
