@@ -1,16 +1,14 @@
 """
 
 """
-import subprocess
-import time
-import sys
 import bisect
 import inspect
 import os
+import sys
 from dataclasses import dataclass
-from functools import wraps, lru_cache
+from functools import lru_cache
+from functools import wraps
 from typing import Any, Dict, Union, Optional, Tuple, Sequence, Mapping
-import random
 
 import numpy as np
 import torch
@@ -18,25 +16,25 @@ from torch import distributed as dist
 from torch import nn
 from torch.optim import Optimizer
 from torch.utils.data import DataLoader
-from functools import lru_cache
+
+from lumo.base_classes import attr
+from lumo.contrib.hug_accelerate import Accelerator
 from lumo.contrib.itertools import safe_cycle
+from lumo.proc.const import TRAINER
+from lumo.proc.dist import is_main, is_dist
+from lumo.utils import dist
+from lumo.utils.device import construct_device_args_kwargs, to_device_enumrate
+from .beans.trainstage import TrainerStage
+from .bundler import DataBundler
+from .datamodule import DataModule
 from .environ import globs
 from .experiment import TrainerExperiment
 from .logger import Logger
-from .saver import Saver
 from .meter import Meter, AvgMeter
-from .rnd import RndManager
-from .datamodule import DataModule
-from .bundler import DataBundler
 from .mixin import ModelMix, CallbackMix, DataModuleMix
-
 from .params import ParamsType
-from lumo.base_classes import attr, TrainerStage
-from lumo.base_classes.metaclasses import Merge
-from ..proc.const import TRAINER
-from lumo.utils import dist
-from lumo.utils.device import construct_device_args_kwargs, to_device_enumrate
-from lumo.contrib.accelerate import Accelerator
+from .rnd import RndManager
+from .saver import Saver
 
 
 @dataclass()
@@ -54,7 +52,7 @@ class TrainerResult(attr):
     MSG_OK_MID = 'success stop in midway'
     MSG_NO_DATALOADER = 'dataloader not loaded'
 
-    def __init__(self, state: TrainerStage, meter: AvgMeter, msg=''):
+    def __init__(self, state: TrainerStage, meter: Optional[Union[Meter, AvgMeter]] = None, msg: str = ''):
         super().__init__()
         self.state = state
         self.meter = meter
@@ -75,6 +73,39 @@ def mp_agent(rank, trainer, op, dataloader):
     op(dataloader)
 
 
+class Merge(type):
+    """
+    元类，用于将子类和父类共有字典，集合时，子类的覆盖行为改为合并父类的字典，集合
+
+    由于用途特殊，仅识别类变量中以下划线开头的变量
+    ::
+        class A(metaclass=Merge):
+            _dicts = {"1": 2, "3": 4}
+
+        class B(A):
+            _dicts = {"5":6,7:8}
+
+        print(B._dicts)
+
+    result:
+    >>> {'5': 6, '3': 4, '1': 2, 7: 8}
+    """
+
+    def __new__(cls, name, bases, attrs: dict, **kwds):
+        for base in bases:
+            for key, value in base.__dict__.items():  # type:(str,Any)
+                if not key.startswith("_"):
+                    continue
+                if isinstance(value, set):
+                    v = attrs.setdefault(key, set())
+                    v.update(value)
+                elif isinstance(value, dict):
+                    v = attrs.setdefault(key, dict())
+                    v.update(value)
+
+        return type.__new__(cls, name, bases, dict(attrs))
+
+
 class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
     """
     1. 组装所有的插件
@@ -94,7 +125,7 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
     """
 
     __exp_name__ = None
-    _call_backs = {
+    callback_function = {
         "save_keypoint", "save_checkpoint", "save_model", "load_state_dict",
         'ioptims', 'icallbacks', 'imodels',
     }
@@ -152,7 +183,7 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
 
         vars = dir(self)
         for name in vars:
-            if name not in self._call_backs:
+            if name not in self.callback_function:
                 continue
             if name.startswith("_"):
                 continue
@@ -164,14 +195,14 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
         return self
 
     @classmethod
-    def _gene_class_exp_name(trainer_instance) -> str:
+    def _gene_class_exp_name(cls) -> str:
         try:
-            file = inspect.getfile(trainer_instance)
+            file = inspect.getfile(cls)
             pre = os.path.splitext(os.path.basename(file))[0]
         except:
             pre = 'builtin'
 
-        return "{}.{}".format(pre.lower(), trainer_instance.__exp_name__.lower())
+        return "{}.{}".format(pre.lower(), cls.__exp_name__.lower())
 
     def __init__(self, params: ParamsType):
         self._state_dicts = attr({
@@ -184,8 +215,8 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
             "others": {},
             'device': torch.device('cpu'),
             'params': params,
-            'exp': TrainerExperiment(self._gene_class_exp_name()),
         })
+        self._exp = TrainerExperiment(self._gene_class_exp_name())
         self._initialize_globs()
         self._initial_exp()
 
@@ -201,7 +232,7 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
 
         if 'device' in params:
             _device = torch.device(params.device)
-            self.regist_device(_device)
+            # self.regist_device(_device)
 
         self._check_cb_init()
         self.exp.start()
@@ -308,7 +339,7 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
 
     @property
     def exp(self) -> TrainerExperiment:
-        return self._state_dicts[TRAINER.DKEY.exp]
+        return self._exp
 
     @property
     def logger(self):
@@ -318,9 +349,9 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
             set_global_logger(self._logger)
             if self.params.get('debug', False):
                 self._logger.set_verbose(Logger.V_DEBUG)
-                self._logger.dddebug('Enable debug log.')
-            fn = self._logger.add_log_dir(self.exp.log_dir)
-            self.exp.writeline('logger', fn)
+                self._logger.debug('Enable debug log.')
+            if is_main():
+                self._logger.add_log_dir(self.exp.log_dir)
         return self._logger
 
     @property
@@ -347,7 +378,7 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
     @property
     def rnd(self) -> RndManager:
         if self._rnd is None:
-            self._rnd = RndManager(self.exp.rnd_dir)
+            self._rnd = RndManager()
         return self._rnd
 
     @property
@@ -358,7 +389,8 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
 
     @property
     def device(self) -> torch.device:
-        return self._state_dicts[TRAINER.DKEY.device]
+        return self.accelerator.device
+        # return self._state_dicts[TRAINER.DKEY.device]
 
     @property
     def device_arg_kwargs(self) -> Tuple[Sequence, dict]:
@@ -490,7 +522,6 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
     def add_callback(self, callback):
         """
         添加一个回调函数，注意，不能添加重复的 callback，这不推荐，也没有必要。
-        :type callable,str
         :param callback:
         :return:
         """
@@ -501,12 +532,12 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
             callback.on_hook_failed(self, msg)
             return
 
-        if callback.only_main_process and self.local_rank > 0:
+        if callback.only_main_process and not is_main():
             msg = f"{callback.__class__.__name__} only_main_process but in local_rank {self.local_rank}"
             callback.on_hook_failed(self, msg)
             return
 
-        if callback.only_single_gpu and self.is_dist:
+        if callback.only_single_gpu and not is_dist():
             msg = f"{callback.__class__.__name__} only_single_gpu but dist={self.is_dist}"
             callback.on_hook_failed(self, msg)
             return
@@ -574,7 +605,8 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
         return {k: v.state_dict() for k, v in self.optim_dict.items()}
 
     def model_state_dict(self):
-        return {k: v.state_dict() for k, v in self.model_dict.items()}
+
+        return {k: self.accelerator.unwrap_model(v).state_dict() for k, v in self.model_dict.items()}
 
     def buffer_state_dict(self):
         return self.buffers
@@ -598,6 +630,7 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
     def _load_fun_state_dict(self, src: dict, tgt: dict):
         for k, v in tgt.items():
             if k in src:
+                v = self.accelerator.unwrap_model(v)
                 v.load_state_dict(src[k])
 
     def ignore_object_state_dict(self, item):
@@ -635,64 +668,37 @@ class _BaseTrainer(ModelMix, CallbackMix, metaclass=Merge):
 
     def save_checkpoint(self, max_keep=10, is_best=False, meta_info: Union[str, dict, Meter] = None):
         info = self._build_trainer_meta_info(meta_info)
-        return self.saver.save_checkpoint(self.eidx, self.state_dict(),
-                                          meta_info=info,
-                                          max_keep=max_keep,
-                                          is_best=is_best)
+        val = self.saver.save_checkpoint(self.eidx, self.state_dict(),
+                                         meta_info=info,
+                                         max_keep=max_keep,
+                                         is_best=is_best)
+        self.wait_for_everyone()
+        return val
+
+    def wait_for_everyone(self):
+        """
+        making sure all processes have reached this point before continuing.
+        """
+        self.accelerator.wait_for_everyone()
 
     def save_keypoint(self, meta_info: Union[str, dict, Meter] = None):
         info = self._build_trainer_meta_info(meta_info)
-        return self.saver.save_keypoint(self.eidx, self.state_dict(),
-                                        meta_info=info)
+        val = self.saver.save_keypoint(self.eidx, self.state_dict(),
+                                       meta_info=info)
+        self.wait_for_everyone()
+        return val
 
     def save_model(self, is_best=False, meta_info: Union[str, dict, Meter] = None):
         info = self._build_trainer_meta_info(meta_info)
-        return self.saver.save_model(self.eidx, {'models': self.model_state_dict()},
-                                     meta_info=info,
-                                     is_best=is_best)
+        val = self.saver.save_model(self.eidx, {'models': self.model_state_dict()},
+                                    meta_info=info,
+                                    is_best=is_best)
+        self.wait_for_everyone()
+        return val
 
 
-class DLLoopMix():
-    def stop_train(self):
-        raise NotImplementedError()
-
-    def stop_train_epoch(self):
-        raise NotImplementedError()
-
-    def to_stage(self, stage: TrainerStage):
-        raise NotImplementedError()
-
-    def train(self, dataloader: Union[DataLoader, DataModule]):
-        raise NotImplementedError()
-
-    def train_epoch(self, dataloader: DataLoader):
-        raise NotImplementedError()
-
-    def train_step(self, idx, batch, *args, **kwargs):
-        raise NotImplementedError()
-
-    def evaluate(self, dataloader: Union[DataLoader, DataModule]):
-        raise NotImplementedError()
-
-    def evaluate_step(self, idx, batch, *args, **kwargs):
-        raise NotImplementedError()
-
-    def test(self, dataloader: Union[DataLoader, DataModule]):
-        raise NotImplementedError()
-
-    def test_step(self, idx, batch, *args, **kwargs):
-        raise NotImplementedError()
-
-    def inference(self, batch):
-        raise NotImplementedError()
-
-    def predict(self, batch):
-        """alias of inference"""
-        raise NotImplementedError()
-
-
-class Trainer(DLLoopMix, _BaseTrainer):
-    _call_backs = {
+class Trainer(_BaseTrainer):
+    callback_function = {
         "train", "train_epoch", "train_step",
         "test", "test_step",
         "evaluate", "evaluate_step",
@@ -700,18 +706,8 @@ class Trainer(DLLoopMix, _BaseTrainer):
         'prepare_dataloader',
     }
 
-    def _wrap_result(self, meter: Union[Mapping, Meter, Sequence, torch.Tensor, np.ndarray]) -> dict:
-        if meter is None:
-            return {}
-        if isinstance(meter, (Mapping, Meter)):
-            return meter
-        elif isinstance(meter, Sequence):
-            return {f'm{i}': v for i, v in enumerate(meter)}
-        elif isinstance(meter, (torch.Tensor, np.ndarray)):
-            return {'metric': meter}
-
     @property
-    def training(self):
+    def training(self) -> bool:
         return self.params.stage == TrainerStage.train.name
 
     def stop_train(self):
@@ -733,14 +729,17 @@ class Trainer(DLLoopMix, _BaseTrainer):
             for k in self._models:
                 v = self._models[k]
                 if not self.accelerator.device_placement:
-                    v.to(self.accelerator.device)
+                    v.to(self.device)
                 v = self.accelerator.prepare(self._models[k])
                 self._models[k] = v
             for k in self._optims:
                 v = self.accelerator.prepare(self._optims[k])
                 self._optims[k] = v
         else:
-            return to_device(item, device_args_kwargs)
+            item = to_device(item, device_args_kwargs)
+            if self.accelerator.use_fp16:
+                item = item.half()
+            return item
 
     def prepare_dataloader(self, stage: TrainerStage, dataloader=None) -> DataLoader:
         """
@@ -755,8 +754,8 @@ class Trainer(DLLoopMix, _BaseTrainer):
         params = self.params
         initialized = getattr(self.initial, f"{stage.name}_dataloader")
 
+        dataloader_ = None
         if dataloader is not None:
-            dataloader_ = None
             """use loader from passed params first, then use itself, finally"""
             if isinstance(dataloader, DataModuleMix):
                 dataloader.iidataloader(params, stage, initialized)
@@ -768,37 +767,31 @@ class Trainer(DLLoopMix, _BaseTrainer):
         elif isinstance(self, DataModuleMix):
             self.iidataloader(params, stage, initialized)
             dataloader_ = getattr(self, f'{stage.name}_dataloader', None)
-
-        if dataloader is None:
-            if initialized:
-                self.datamodule.iidataloader(params, stage, initialized)
+        else:
+            self.datamodule.iidataloader(params, stage, initialized)
             dataloader_ = getattr(self.datamodule, f'{stage.name}_dataloader', None)
 
-        if isinstance(dataloader_, DataLoader):
-            dataloader_ = self.accelerator.prepare(dataloader_)
-        elif isinstance(dataloader_, DataBundler):
-            for k, (loader, func) in list(dataloader_.dataloaders.items()):
-                dataloader_.dataloaders[k] = [self.accelerator.prepare(loader), func]
+        if not initialized and stage.is_train:
+            if isinstance(dataloader_, DataLoader):
+                dataloader_ = self.accelerator.prepare_data_loader(dataloader_)
+            elif isinstance(dataloader_, DataBundler):
+                for k, (loader, func) in list(dataloader_.dataloaders.items()):
+                    dataloader_.dataloaders[k] = [self.accelerator.prepare(loader), func]
 
-        kwargs = {stage.name: dataloader_}
-        self.regist_dataloader(**kwargs)
+            kwargs = {stage.name: dataloader_}
+            self.regist_dataloader(**kwargs)
         return dataloader_
 
     def train(self, dataloader: Union[DataLoader, DataModuleMix] = None) -> TrainerResult:
-
         dataloader = self.prepare_dataloader(TrainerStage.train, dataloader)
         if dataloader is None:
-            return TrainerResult(TrainerStage.train, 1, 'no train_dataloader')
-
-        self._check_models_init()
-        self._check_optim_init()
+            return TrainerResult(TrainerStage.train, None, 'no train_dataloader')
 
         self.initial.train_dataloader = True
-
         result = TrainerResult(TrainerStage.train, None, TrainerResult.MSG_OK)
+
         while self.params.eidx < self.params.epoch:
             self.params.eidx += 1
-            self.to_stage(TrainerStage.train)
             result = self.train_epoch(dataloader)
             if self.train_toggle:
                 self.train_toggle = False
@@ -815,6 +808,7 @@ class Trainer(DLLoopMix, _BaseTrainer):
 
         self._check_models_init()
         self._check_optim_init()
+        self.change_mode(False)
 
         self.initial.train_dataloader = True
 
@@ -826,7 +820,7 @@ class Trainer(DLLoopMix, _BaseTrainer):
             self.params.idx = idx
             batch = next(loader)
             meter = self.train_step(idx, batch, self.params)
-            avg.update(self._wrap_result(meter))
+            avg.update(Meter.wrap_result(meter))
             idx += 1
             if self.train_epoch_toggle:
                 self.train_epoch_toggle = False
@@ -834,6 +828,10 @@ class Trainer(DLLoopMix, _BaseTrainer):
         return TrainerResult(TrainerStage.train_epoch, avg, TrainerResult.MSG_OK)
 
     def train_epoch(self, dataloader: DataLoader) -> TrainerResult:
+        self._check_models_init()
+        self._check_optim_init()
+        self.change_mode(True)
+
         avg = None
         for idx, batch in enumerate(dataloader):
             if avg is None:
@@ -841,28 +839,30 @@ class Trainer(DLLoopMix, _BaseTrainer):
             self.params.global_step += 1
             self.params.idx = idx
             meter = self.train_step(idx, batch, self.params)
-            avg.update(self._wrap_result(meter))
+            avg.update(Meter.wrap_result(meter))
             if self.train_epoch_toggle:
                 self.train_epoch_toggle = False
                 break
         return TrainerResult(TrainerStage.train_epoch, avg, TrainerResult.MSG_OK)
 
-    def train_step(self, idx, batch, params: ParamsType, *args, **kwargs):
+    def train_step(self, idx, batch, params: ParamsType, *args, **kwargs) -> Meter:
         pass
 
     def evaluate(self, dataloader: Union[DataLoader, DataModule] = None) -> TrainerResult:
         dataloader = self.prepare_dataloader(TrainerStage.val, dataloader)
         if dataloader is None:
             return TrainerResult(TrainerStage.val, None, TrainerResult.MSG_NO_DATALOADER)
+        self.initial.val_dataloader = True
+
         self._check_models_init()
-        self.to_stage(TrainerStage.val)
+        self.change_mode(False)
 
         with torch.no_grad():
             avg = AvgMeter()
             for idx, batch in to_device_enumrate(dataloader, self.device_arg_kwargs):
                 self.params.idx = idx
                 meter = self.evaluate_step(idx, batch, self.params)
-                avg.update(self._wrap_result(meter))
+                avg.update(Meter.wrap_result(meter))
 
         return TrainerResult(TrainerStage.val, avg, TrainerResult.MSG_OK)
 
@@ -870,22 +870,24 @@ class Trainer(DLLoopMix, _BaseTrainer):
         pass
 
     def test(self, dataloader: Union[DataLoader, DataModule] = None) -> TrainerResult:
-
         dataloader = self.prepare_dataloader(TrainerStage.test, dataloader)
         if dataloader is None:
             return TrainerResult(TrainerStage.test, None, TrainerResult.MSG_NO_DATALOADER)
+
+        self.initial.test_dataloader = True
+
         self._check_models_init()
-        self.to_stage(TrainerStage.test)
+        self.change_mode(False)
 
         with torch.no_grad():
             avg = AvgMeter()
             for idx, batch in to_device_enumrate(dataloader, self.device_arg_kwargs):
                 self.params.idx = idx
                 meter = self.test_step(idx, batch, self.params)
-                avg.update(self._wrap_result(meter))
+                avg.update(Meter.wrap_result(meter))
         return TrainerResult(TrainerStage.test, avg, TrainerResult.MSG_OK)
 
-    def test_step(self, idx, batch, params: ParamsType, *args, **kwargs):
+    def test_step(self, idx, batch, params: ParamsType, *args, **kwargs) -> Meter:
         pass
 
     def inference(self, batch):
@@ -903,3 +905,72 @@ class Trainer(DLLoopMix, _BaseTrainer):
 
     def imodels(self, params: ParamsType):
         pass
+
+
+class FitTrainer(Trainer):
+    def __init__(self, params: ParamsType, model, loss_fn, optim, callbacks=None):
+        super().__init__(params)
+        from lumo.nest.trainer.model import FitModel
+
+        self.model: FitModel = model
+        self.loss_fn = loss_fn
+        self.optim = optim
+        self.to_device()
+        if callbacks is not None:
+            from .callbacks import BaseCallback
+            if isinstance(callbacks, BaseCallback):
+                callbacks = [callbacks]
+            for cb in callbacks:
+                cb.hook(self)
+
+    def train_step(self, idx, batch, params: ParamsType, *args, **kwargs) -> Meter:
+        meter = Meter()
+
+        losses, meters = self.model.feed(batch, loss_fn=self.loss_fn)
+        losses = list(losses.values())
+        loss = sum(losses)
+
+        for k, v in losses.items():
+            meter.mean[k] = v
+
+        for k, v in meters.items():
+            meter.mean[k] = v
+
+        self.optim.zero_grad()
+        self.accelerator.backward(loss)
+        self.optim.step()
+        meter.mean.Lall = loss
+        return meter
+
+    def evaluate_step(self, idx, batch, params: ParamsType, *args, **kwargs) -> Meter:
+        meter = Meter()
+
+        losses, meters = self.model.feed(batch, loss_fn=self.loss_fn)
+        losses = list(losses.values())
+
+        for k, v in losses.items():
+            meter.mean[k] = v
+
+        for k, v in meters.items():
+            meter.mean[k] = v
+        if len(losses) > 0:
+            loss = sum(losses)
+            meter.mean.Lall = loss
+        return meter
+
+    def test_step(self, idx, batch, params: ParamsType, *args, **kwargs) -> Meter:
+        meter = Meter()
+
+        losses, meters = self.model.feed(batch, loss_fn=self.loss_fn)
+        losses = list(losses.values())
+
+        for k, v in losses.items():
+            meter.mean[k] = v
+
+        for k, v in meters.items():
+            meter.mean[k] = v
+
+        if len(losses) > 0:
+            loss = sum(losses)
+            meter.mean.Lall = loss
+        return meter
