@@ -1,10 +1,16 @@
 import os
 import sys
 
+from lumo.utils.repository import git_dir
+
 
 def cache_dir():
-    res = os.path.expanduser("~/.cache/lumo")
-    os.makedirs(res, exist_ok=True)
+    try:
+        res = os.path.expanduser("~/.cache/lumo")
+        os.makedirs(res, exist_ok=True)
+    except PermissionError:
+        res = os.path.expanduser("~/.lumo/.cache")
+        os.makedirs(res, exist_ok=True)
     return res
 
 
@@ -19,28 +25,6 @@ def libhome():
     return os.path.expanduser("~/.lumo")
 
 
-def git_dir(root='./'):
-    """
-    git repository directory
-    Args:
-        root:
-
-    Returns:
-
-    """
-    from lumo.proc.explore import git_enable
-    if git_enable():
-        from git import Git
-        cur = os.getcwd()
-        os.chdir(root)
-        res = Git().execute(['git', 'rev-parse', '--git-dir'])
-        res = os.path.abspath(os.path.dirname(res))
-        os.chdir(cur)
-        return res
-    else:
-        return None
-
-
 def local_dir():
     res = git_dir()
     if res is None:
