@@ -1,27 +1,9 @@
 """
 Methods about files/paths
 """
-from lumo.utils import re
 import os
 
-
-def checkpath(*path, is_file=False):
-    """
-
-    Args:
-        *path:
-
-    Returns:
-
-    """
-    res = os.path.join(*path)
-    if is_file:
-        os.makedirs(os.path.dirname(res), exist_ok=True)
-    else:
-        os.makedirs(res, exist_ok=True)
-    if not os.path.exists(res):
-        return None
-    return res
+from . import re
 
 
 def compare_path(a, b):
@@ -61,3 +43,7 @@ def filter_filename(title: str, substr='-'):
     """Create a valid path by replacing invalid character into the given `substr`."""
     title = re.sub('[\/:*?"<>|]', substr, title)
     return title
+
+
+def safe_filename(fn, sep='_'):
+    return re.sub('[\/:*?"<>+\-|]', sep, fn)
