@@ -1,12 +1,14 @@
 """
 Methods about git.
 """
-import git
 import os
+import warnings
 from functools import lru_cache
+
 from lumo.base_classes import attr
 
 try:
+    import git
     from git import Repo, Commit
 except ImportError:
     def Repo(*_, **__):
@@ -16,10 +18,6 @@ except ImportError:
     def Commit(*_, **__):
         return None
 
-from lumo.proc.explore import git_enable
-from lumo.proc.path import git_dir, cache_dir
-from lumo.utils.paths import checkpath
-
 
 class GitKey:
     LUMO_BRANCH = 'lumo_experiments'
@@ -153,11 +151,18 @@ class GitEnabledWrap(GitWrap):
                 return _commits_map[key]
 
             with branch(repo, branch_name):
-                repo.git.add(all=True)
-                commit_info = '[[EMPTY]]'
-                if info is not None:
-                    commit_info = info
-                commit_ = repo.index.commit(commit_info)
+                change_file = []
+                change_file.extend(repo.untracked_files)
+                change_file.extend([i.a_path for i in repo.index.diff('Head')])
+                change_file.extend([i.a_path for i in repo.index.diff(None)])
+                if len(change_file) > 0:
+                    repo.git.add(change_file)
+                    commit_info = '[[EMPTY]]'
+                    if info is not None:
+                        commit_info = info
+                    commit_ = repo.index.commit(commit_info)
+                else:
+                    commit_ = repo.head.commit
             if key is not None:
                 _commits_map[key] = commit_
         except git.GitCommandError:
@@ -188,7 +193,6 @@ class GitEnabledWrap(GitWrap):
         os.chdir(commit.tree.abspath)
         # exp = Experiment('Reset')
 
-        from thexp.utils.repository import branch
         with branch(commit.repo, GitKey.LUMO_BRANCH) as new_branch:
             repo.git.checkout(commit.hexsha)
             repo.git.checkout('-b', 'reset')
@@ -196,13 +200,6 @@ class GitEnabledWrap(GitWrap):
             repo.git.add('.')
             ncommit = repo.index.commit("Reset from {}".format(commit.hexsha))
             repo.git.branch('-d', 'reset')
-        # exp.add_plugin('reset', {
-        #     'test_name': self.name,  # 从哪个状态恢复
-        #     'from': exp.commit.hexsha,  # reset 运行时的快照
-        #     'where': commit.hexsha,  # 恢复到哪一次 commit，是恢复前的保存的状态
-        #     'to': ncommit.hexsha,  # 对恢复后的状态再次进行提交，此时 from 和 to 两次提交状态应该完全相同
-        # })
-        # exp.end()
 
         os.chdir(old_path)
         return None
@@ -222,20 +219,40 @@ class GitEnabledWrap(GitWrap):
         os.chdir(commit.tree.abspath)
         # exp = Experiment('Archive')
 
-        revert_path = checkpath(cache_dir(), 'archives', commit)
-        revert_fn = os.path.join(revert_path, "code.zip")
+        # revert_path = checkpath(cache_dir(), 'archives', commit)
+        # revert_fn = os.path.join(revert_path, "code.zip")
 
         # TODO 在code.zip目录下添加相关说明
         # exp.add_plugin('archive', {'file': revert_fn,
         #                            'test_name': self.name})
-        with open(revert_fn, 'wb') as w:
-            repo.archive(w, commit)
+        # with open(revert_fn, 'wb') as w:
+        #     repo.archive(w, commit)
 
         # exp.end()
         os.chdir(old_path)
         return None
 
 
+@lru_cache(1)
+def git_enable():
+    git_enabled = (os.environ.get('LUMO_GIT', '1') == '1' and
+                   os.environ.get('LUMO_NOGIT', '0') == '0')
+    if not git_enabled:
+        return False
+
+    try:
+        import git
+    except ImportError:
+        warnings.warn('python library `gitpython` not installed, git operations will be ignored. '
+                      'If you want lumo to use git to manage your code, please install it by executing `pip install gitpython`')
+        return False
+    try:
+        git.Git().execute(['git', 'rev-parse', '--git-dir'])
+        return True
+    except git.GitCommandError:
+        return False
+
+
 if git_enable():
     wrap = GitEnabledWrap()
 else:
@@ -247,3 +264,24 @@ reset = wrap.reset
 archive = wrap.archive
 
 _commits_map = {}
+
+
+def git_dir(root='./'):
+    """
+    git repository directory
+    Args:
+        root:
+
+    Returns:
+
+    """
+    if git_enable():
+        from git import Git
+        cur = os.getcwd()
+        os.chdir(root)
+        res = Git().execute(['git', 'rev-parse', '--git-dir'])
+        res = os.path.abspath(os.path.dirname(res))
+        os.chdir(cur)
+        return res
+    else:
+        return None
